/*
    Copyright (c) 2015 Guadmaz

    Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
    associated documentation files (the "Software"), to deal in the Software without restriction, including
    without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the
    following conditions:

    The above copyright notice and this permission notice shall be included in all copies or substantial portions
    of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
    TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
    CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.
*/
using System;
using System.Windows.Forms;
using GTA;
using GTA.Native;
using GTA.Math;

public class GrabAnything : GTA.Script
{
    public GrabAnything()
    {
        Tick += delegate(object sender, EventArgs e)
        {
            var player = Game.Player.Character;
        
            if (Game.IsControlPressed(0, GTA.Control.FrontendLb) && !_qDown)
            {
                var coord = player.GetBoneCoord(Bone.SKEL_L_Hand);
                
                var res = RaycastSphere(coord, 0.25f);
                if (res.HitAnything)
                {
                    _qDown = true;
                    
                    var rightPos = coord;
                    _leftPos = coord;
                    
                    if (_rightPos != new Vector3()) rightPos = _rightPos;
                    
                    player.Euphoria.Grab.Pos1 = rightPos; // R Hand
                    player.Euphoria.Grab.Pos2 = _leftPos; // L Hand
                    player.Euphoria.Grab.UseLeft = true;
                    player.Euphoria.Grab.UseRight = _eDown;
                    player.Euphoria.Grab.DropWeaponIfNecessary = true;
                    player.Euphoria.Grab.DropWeaponDistance = 5f;
                    player.Euphoria.Grab.LookAtGrab = true;
                    player.Euphoria.Grab.GrabStrength = 1000.0f;
                    player.Euphoria.Grab.GrabDistance = 4.0f;

                    player.Euphoria.Grab.FromEA = true;
                    player.Euphoria.Grab.ArmStiffness = 100f;
                    player.Euphoria.Grab.BodyStiffness = 100f;
                    player.Euphoria.Grab.MaxReachDistance = 10f;

                    player.Euphoria.Grab.GrabHoldMaxTimer = 20f;// in seconds (float)
                    player.Euphoria.Grab.DontLetGo = true;
                    player.Euphoria.Grab.Start(20000);
                    player.Euphoria.PedalLegs.Start();
                }
            }
            
            if (Game.IsControlPressed(0, GTA.Control.FrontendRb) && !_eDown)
            {
                var coord = player.GetBoneCoord(Bone.SKEL_R_Hand);
                
                var res = RaycastSphere(coord, 0.25f);
                if (res.HitAnything)
                {
                    _eDown = true;
                    
                    var leftPos = coord;
                    var _rightPos = coord;
                    
                    if (_leftPos != new Vector3()) leftPos = _leftPos;
                    
                    player.Euphoria.Grab.Pos1 = _rightPos; // R Hand
                    player.Euphoria.Grab.Pos2 = leftPos; // L Hand
                    player.Euphoria.Grab.UseLeft = _qDown;
                    player.Euphoria.Grab.UseRight = true;
                    player.Euphoria.Grab.DropWeaponIfNecessary = true;
                    player.Euphoria.Grab.DropWeaponDistance = 5f;
                    player.Euphoria.Grab.LookAtGrab = true;
                    player.Euphoria.Grab.GrabStrength = 1000.0f;
                    player.Euphoria.Grab.GrabDistance = 4.0f;

                    player.Euphoria.Grab.FromEA = true;
                    player.Euphoria.Grab.ArmStiffness = 100f;
                    player.Euphoria.Grab.BodyStiffness = 100f;
                    player.Euphoria.Grab.MaxReachDistance = 10f;

                    player.Euphoria.Grab.GrabHoldMaxTimer = 20f;// in seconds (float)
                    player.Euphoria.Grab.DontLetGo = true;
                    player.Euphoria.Grab.Start(20000);
                    player.Euphoria.PedalLegs.Start();
                }
            }
            
            if (Game.IsControlJustReleased(0, GTA.Control.FrontendLb) && _qDown)
            {
                _qDown = false;
                _leftPos = new Vector3();
                if (!_eDown)
                {
                    Game.Player.Character.Euphoria.StopAllBehaviours.Start(1000);
                    Wait(1000);
                    Game.Player.Character.Euphoria.StopAllBehaviours.Stop();
                    Game.Player.Character.Euphoria.Grab.Stop();
                    Game.Player.Character.Euphoria.PedalLegs.Stop();
                }
            }
            
            if (Game.IsControlJustReleased(0, GTA.Control.FrontendRb) && _eDown)
            {
                _eDown = false;
                _rightPos = new Vector3();
                if (!_qDown)
                {
                    Game.Player.Character.Euphoria.StopAllBehaviours.Start(1000);
                    Wait(1000);
                    Game.Player.Character.Euphoria.StopAllBehaviours.Stop();
                    Game.Player.Character.Euphoria.Grab.Stop();
                    Game.Player.Character.Euphoria.PedalLegs.Stop();
                }
            }
        };
    }
    
    private bool _qDown;
    private bool _eDown;
    
    private Vector3 _leftPos;
    private Vector3 _rightPos;
       
    public SphereRaycastResult RaycastSphere(Vector3 position, float radius)
    {
        var result = World.RaycastCapsule(position - new Vector3(0,0,radius),
                position + new Vector3(0,0,radius), radius,
                IntersectOptions.Map | IntersectOptions.Objects | IntersectOptions.Vegetation,
                Game.Player.Character);
        
        var output = new SphereRaycastResult();
        output.HitAnything = result.DitHitAnything;
        output.HitEntity = result.DitHitEntity;
        
        output.WorldCoords = result.HitCoords;
        output.EntityResult = result.HitEntity;
        return output;
    }
}

public class SphereRaycastResult
{
    public Entity EntityResult { get; set; }
    public Vector3 WorldCoords { get; set; }
    
    public bool HitAnything { get; set; }
    public bool HitEntity { get; set; }
}